!+
! Module: omthconv_mod
!
! Authors:  jfr - MSSL
!
! Description: Converts tracking history data into count-rates
!
! Comments:
!
! $Id: module.f90,v 1.5 1998/01/13 08:39:15 gvacanti Exp $
!-
! History:
!          1998 MAY: created jfr
!          1999 Jan: updated psw - spr 556
!          1999 Mar: updated jfr
!          1999 May: updated jfr
!          2000 July: Ammended AMK
!          Added additional code to identify the exposure number and return the start and end times
!          from the new global HK files.
!
!          2000 JUly: Ammended CB - removed bug connected with opening file.
!          2000 August: Ammended AMK - changed parameters in call message
!
!          2000 October: Ammended AMK - changed call to getstartendexp with the extra parameter filterString.
!          This parameter is not used but must be present for the call to succeed.
!          03-05-2001  (CB) - Ammended call to getstartendexp- added arguement instrumentMode 
!          24-05-2001 (VNY) - The following names of the output table parameters have been changed:
!                             RA_PNT -> RA_SCX, DEC_PNT -> DEC_SCX, PA_PNT -> PA
!          28-09-2001 (VNY) - obtTags is already in the correct format of time (not Tags). Therefore, 
!                             there is no necessity to convert it to time using the call to  
!                             OAL_obtToTimeTag(obtTags, timeTagsP). This call is replaced with 
!                             a simple asiggment statement
!-           
!**************************************************************************
module omthconv_mod

  use param
  use oal
  use caloaldefs
  use dal
  use errorHandling
  use getstartendexp_mod 

  implicit none

  private

  public omthconv

contains

  !---------------------------------------------------------------------
  subroutine omthconv
  !---------------------------------------------------------------------

    character(len=255) :: thxSet                                      ! OM ODF Tracking History Data Auxiliary filename.
    character(len=255) :: nphSet                                      ! OM ODF Non-Periodic Housekeeping filename.
    character(len=255) :: outSet                                      ! OM PPS FITS Tracking Star Timeseries filename.
    type(DataSetT) :: thxDataSetH                                     ! Tracking History Data dataset handle.
    type(DataSetT) :: outDataSetH                                     ! Output dataset handle.

    type(TableT) :: thxTableH                                         ! Tracking History Data table dataset handle.
    type(TableT) :: outTableH                                         ! Output table handle.

    type(ColumnT), dimension(2) :: thxColumnH                         ! Tracking History Data column handles.
    type(ColumnT), dimension(10) :: outColumnH                        ! Column handles.

    integer(kind=int32), dimension(:,:), pointer :: countsP           ! Counts column pointer.
!    real(kind=double), dimension(:), pointer :: timeTagsP             ! Event time tags column pointer.
    real(kind=double), dimension(0:1) :: timeTagsP             ! Event time tags column pointer.
    real(kind=single), dimension(:), pointer :: R32ColumnP            ! Column pointer.

    character(len=255) :: dataMode                                    ! Input file data mode.
    integer(kind=int32) :: nFrames                                    ! Number of rows in binary table extension.
    integer(kind=int32) :: frmTime                                    ! Keywords.
    real(kind=double) :: OBTstart                                     ! Start time.
    real(kind=double) :: OBTend                                       ! End time.
    real(kind=double), dimension(0:1) :: obtTags                      ! Start/end OBT tag array

    integer(kind=int32), dimension(1) :: shapeDim                     ! Column Dimension.
    integer(kind=int32) :: i                                          ! Counter
    integer :: modeFlag
    integer :: instrumentMode 
    !
    !-------------------------------------------------------------------------------------------------------
    !
    ! AMK - 15/07/2000
    !
    ! Extra defines
    !
    character(len=19)            :: dateObs              ! Dummy just to satisfy getstartendexp
    character(len=19)            :: dateEnd              ! Dummy just to satisfy getstartendexp
    character(len=255)           :: obsId                ! Keywords.
    character(len=255)           :: expId                ! Keywords.

    character(len=3)             :: exposureNumber       ! String to hold the exposure number. 
    character(len=32)            :: filterString         ! Exposure Filter String - not used but for consistency.

    integer(kind=single)         :: length1              ! String length.
    integer(kind=single)         :: length2              ! String length. 
    integer                      :: expNumber            ! Actual exposure number

    !-------------------------------------------------------------------------------------------------------
    !

    ! Read in parameters.

    thxSet = stringParameter("thxset")
    nphSet = stringParameter("nphset")
    outSet = stringParameter("outset")
    thxDataSetH = dataSet(thxset, read)


!    slewFlag = booleanParameter("slewset")
    modeFlag = intParameter("modeset")   ! Tells omthconv what data mode is (IMAGING (0), FAST MODE (1) or SLEW (2) mode)


    !****************************************
    ! Set the instrument mode from modeFlag - 
    ! added on 4th May 2001 to get the fast-chain working
    !****************************************
    select case(modeFlag)
        case(0)
            instrumentMode = IMAGING
        case(1) 
            instrumentMode = DATA_FAST
        case default
            instrumentMode = IMAGING
    end select



    if (hasBlock(thxDataSetH, "ZERODRIFT")) then
       write (errstr, *) "Missing OM Tracking History Data Auxiliary File. Using ZERODRIFT option"      
       write (errstr,*) "No PPS Tracking History Products available" 
       call warning("ZERODRIFT")
    else

       ! Check data types of ODF input files

       ! Get a handle on the ODF OM Tracking History Data Auxiliary File.

       !********************************************************************************
       ! Determine datamode from keyword in primary header.

      dataMode = trim(stringAttribute(thxDataSetH, "DATAMODE")) 
      if (dataMode == "TRACKING") then
          ! Get a handle on the tracking history data binary table extension.

          thxTableH = table(thxDataSetH, "OMTHX1")
        
          !*******************************************************************************
          !
          ! AMK - 25/07/2000 - Added
          obsId = stringAttribute(thxTableH, "OBS_ID")
          expId = stringAttribute(thxTableH, "EXP_ID")
          !*******************************************************************************
          nFrames = numberOfRows(thxTableH)
          frmTime = int32Attribute(thxTableH, "FRMTIME")

          thxColumnH(1) = column(thxTableH, "GSCTS", read)
          countsP => int32VectorData(thxColumnH(1))


          ! Get start and end of exposure OBT time stamps from the ODF Non-Periodic Housekeeping File.
          !
          ! AMK - 14/07/2000
          ! Removed the call below and inserted new routine below
          !

          !
          ! Get lengths of the strings holding the observation ID and the exposure Id.
          !
          length1 = len_trim(expId)
          length2 = len_trim(obsId)

          !
          ! Set the string for the exposure number
          !
          exposureNumber = expId(length2 + 1 : length1)

          !
          ! Now convert the string to an integer
          !

          read(exposureNumber,'(i3)') expNumber
          call getstartendexp(nphSet,  expNumber, instrumentMode, OBTstart, OBTend, dateObs, dateEnd, filterString)

          ! Verbosity info.

          write (errstr, *) OBTstart, OBTend
          call message(AppMsg,Verbose)

          ! Convert Onboard start/end Timestamps (OBT) to event time tags (elapsed seconds since mission 
          ! reference time 1998-01-01T00:00:00).
          obtTags(0) = OBTstart
          obtTags(1) = OBTend - real(frmTime) / 1024.0

          call OAL_setState(OM)
!---  VNY 28.09.2001 obtTags is already is in the correct format of time (not Tags). Therefore, 
!                    there is no necessity to convert it to time. Replaces by a simple assignment 
!          call OAL_obtToTimeTag(obtTags, timeTagsP)
          timeTagsP=obtTags
!---
          write(errstr,*) "TIME TAGS", timeTagsP(0), timeTagsP(1)
          call message(AppMsg,Verbose)
          outDataSetH = dataSet(outSet, create)
   
          call setAttribute(outDataSetH, "TELESCOP", stringAttribute(thxDataSetH, "TELESCOP"), "Telescope (mission) name")
          call setAttribute(outDataSetH, "INSTRUME", stringAttribute(thxDataSetH, "INSTRUME"), "Instrument name")
          call setAttribute(outDataSetH, "DETNAM", stringAttribute(thxDataSetH, "DETNAM"), "PRIME or REDUNDANT")
          call setAttribute(outDataSetH, "DATAMODE","TRACKING" , "Instrument mode")
          call setAttribute(outDataSetH, "FILTER", stringAttribute(thxDataSetH, "FILTER"), "Filter ID")
          call setAttribute(outDataSetH, "OBS_MODE",stringAttribute(thxDataSetH, "OBS_MODE"), "Observation mode (pointing or slew)")
          call setAttribute(outDataSetH, "OBS_ID", stringAttribute(thxDataSetH, "OBS_ID"), "Observation identifier")
          call setAttribute(outDataSetH, "EXP_ID", stringAttribute(thxDataSetH, "EXP_ID"), "Exposure identifier")
          call setAttribute(outDataSetH, "CONTENT", "OM TRACKING STAR TIMESERIES", "Content of file")
          call setAttribute(outDataSetH, "ORIGIN", stringAttribute(thxDataSetH, "ORIGIN"), "origin of FITS file")
          call setAttribute(outDataSetH, "OBJECT", stringAttribute(thxDataSetH, "OBJECT"), "Name of observed object")

!          call setAttribute(outDataSetH, "RA_PNT", real32Attribute(thxDataSetH, "RA_PNT"), "RA of instrument pointing")
!          call setAttribute(outDataSetH, "DEC_PNT", real32Attribute(thxDataSetH, "DEC_PNT"), "DEC of instrument pointing")
!          call setAttribute(outDataSetH, "PA_PNT", real32Attribute(thxDataSetH, "PA_PNT"), "ROLL angle of instrument pointing")
          call setAttribute(outDataSetH, "RA_SCX", real32Attribute(thxDataSetH, "RA_SCX"), "RA of instrument pointing")
          call setAttribute(outDataSetH, "DEC_SCX", real32Attribute(thxDataSetH, "DEC_SCX"), "DEC of instrument pointing")
          call setAttribute(outDataSetH, "PA", real32Attribute(thxDataSetH, "PA"), "ROLL angle of instrument pointing")

          call setAttribute(outDataSetH, "RADECSYS", stringAttribute(thxDataSetH, "RADECSYS"), " ")
          call setAttribute(outDataSetH, "EQUINOX", real32Attribute(thxDataSetH, "EQUINOX"), "Equinox of celestial coord system")

          call setAttribute(outDataSetH, "EXPOSURE", real32Attribute(thxDataSetH, "EXPOSURE"), "Exposure (secs)")
          call setAttribute(outDataSetH, "DATE-OBS", stringAttribute(thxDataSetH, "DATE-OBS"), "Date observations were made")
          call setAttribute(outDataSetH, "DATE-END", stringAttribute(thxDataSetH, "DATE-END"), "Date observations ended")
          call setAttribute(outDataSetH, "OBSERVER", stringAttribute(thxDataSetH, "OBSERVER"), "Name of PI")
          call setAttribute(outDataSetH, "TIMESYS", "TT", "fundamental time system: Terrestial Time")
          call setAttribute(outDataSetH, "TIMEUNIT", "s", "unit for TSTART, TSTOP, TIMEZERO, TELPASE")
          call setAttribute(outDataSetH, "TASSIGN", "SATELLITE", "location of time assignment")
          call setAttribute(outDataSetH, "TIMEREF", "LOCAL", "spatial reference frame")
          call setAttribute(outDataSetH, "CLOCKAPP", "T","times corrected for clock drifts?")
! VNY 28.09.2001 Set TIMEZERO attribute to its real value
!          call setAttribute(outDataSetH, "TIMEZERO", 0, "s", "obs. start time relative to MJDREF")
          call setAttribute(outDataSetH, "TIMEZERO", OBTstart, "s", "obs. start time relative to MJDREF")
          call setAttribute(outDataSetH, "MJDREF", 50814.0, "d", "MJD of 1998-01-01T00:00:00")

          ! Create and get a handle on RATES extension.

          outTableH = addTable(outDataSetH, "RATES", nFrames)

          !write(*,*)'GOT HANDLE TO OUTPUT TABLE'

          ! Write attributes.

          call setAttribute(outTableH, "HDUCLASS", "OGIP", "Format conforms to OGIP/GSFC conventions")
          call setAttribute(outTableH, "HDUCLAS1", "LIGHTCURVE", "File contains a time-series")
          call setAttribute(outTableH, "HDUCLAS2", "TOTAL", "Not background subtracted")
          call setAttribute(outTableH, "HDUCLAS3", "RATE", "Not exposure corrected")
          call setAttribute(outTableH, "TIMEDEL", REAL(frmTime)/1024.0, " Light curve bin size")
          call setAttribute(outTableH, "TSTART", timeTagsP(0), " Light curve start time")
          call setAttribute(outTableH, "TSTOP", timeTagsP(1), " Light curve stop time")
          call setAttribute(outTableH, "TIMEUNIT", "S", "unit for TSTART, TSTOP, TIMEZERO, TELPASE")

          ! Write RATEn columns.

          shapeDim(1) = 1

          outColumnH(1) = addColumn(outTableH, "RATE1", real32,"count/s",shapeDim)
         
          R32ColumnP => real32data(outColumnH(1))
          do i = 0 , nFrames - 1
             R32ColumnP(i) = REAL(countsP(0, i)) /  (REAL(frmTime) / 1024.0)
          end do

          outColumnH(2) = addColumn(outTableH, "RATE2", real32,"count/s",shapeDim)
          
          R32ColumnP => real32data(outColumnH(2))
          do i = 0 , nFrames - 1
             R32ColumnP(i) = REAL(countsP(1, i) / (REAL(frmTime) / 1024.0))
          end do

          outColumnH(3) = addColumn(outTableH, "RATE3", real32,"count/s",shapeDim)
          
          R32ColumnP => real32data(outColumnH(3))
          do i = 0 , nFrames - 1
             R32ColumnP(i) = REAL(countsP(2, i) / (REAL(frmTime) / 1024.0))
          end do

          outColumnH(4) = addColumn(outTableH, "RATE4", real32,"count/s",shapeDim)
          
          R32ColumnP => real32data(outColumnH(4))
          do i = 0 , nFrames - 1
             R32ColumnP(i) = REAL(countsP(3, i) / (REAL(frmTime) / 1024.0))
          end do

          outColumnH(5) = addColumn(outTableH, "RATE5", real32,"count/s",shapeDim)
          
          R32ColumnP => real32data(outColumnH(5))
          do i = 0 , nFrames - 1
             R32ColumnP(i) = REAL(countsP(4, i) / (REAL(frmTime) / 1024.0))
          end do

          outColumnH(6) = addColumn(outTableH, "RATE6", real32,"count/s",shapeDim)
          
          R32ColumnP => real32data(outColumnH(6))
          do i = 0 , nFrames - 1
             R32ColumnP(i) = REAL(countsP(5, i) / (REAL(frmTime) / 1024.0))
          end do

          outColumnH(7) = addColumn(outTableH, "RATE7", real32,"count/s",shapeDim)
         
          R32ColumnP => real32data(outColumnH(7))
          do i = 0 , nFrames - 1
             R32ColumnP(i) = REAL(countsP(6, i) / (REAL(frmTime) / 1024.0))
          end do

          outColumnH(8) = addColumn(outTableH, "RATE8", real32,"count/s",shapeDim)
          
          R32ColumnP => real32data(outColumnH(8))
          do i = 0 , nFrames - 1
             R32ColumnP(i) = REAL(countsP(7, i) / (REAL(frmTime) / 1024.0))
          end do

          outColumnH(9) = addColumn(outTableH, "RATE9", real32,"count/s",shapeDim)
       
          R32ColumnP => real32data(outColumnH(9))
          do i = 0 , nFrames - 1
             R32ColumnP(i) = REAL(countsP(8, i) / (REAL(frmTime) / 1024.0))
          end do

          outColumnH(10) = addColumn(outTableH, "RATE10", real32,"count/s",shapeDim)
         
          R32ColumnP => real32data(outColumnH(10))
          do i = 0 , nFrames - 1
             R32ColumnP(i) = REAL(countsP(9, i) / (REAL(frmTime) / 1024.0))
          end do

          ! Free up memory and pointers.

          ! Release handles on data.

!          call OAL_releaseMemory(timeTagsP)
          
          call release(outColumnH(10))
          call release(outColumnH(9))
          call release(outColumnH(8))
          call release(outColumnH(7))
          call release(outColumnH(6))
          call release(outColumnH(5))
          call release(outColumnH(4))
          call release(outColumnH(3))
          call release(outColumnH(2))
          call release(outColumnH(1))
          call release(thxColumnH(1))
          call release(outTableH)
          call release(thxTableH)
          call release(outDataSetH)
 
       else

          ! Incompatible file.

          call fatal("badFileMode","Value for keyword DATAMODE is incompatible")

       end if

    end if
       
    call release(thxDataSetH)
    
  end subroutine omthconv

end module omthconv_mod















